/*************************************************************************
 * File Name     : QUERY.C
 *
 * Description   : This sample program demonstrates how to use the API
 *                 LayoutQueryValues.
 *                 This an API that is used to query  the current
 *                 setting of layout values within a LayoutObject.
 *
 * Concepts      : The layout values in the LayoutTextDescriptor describe
 *                 both the input and output strings.
 *                 The API does the necessary to get there values

 *
 * API's         : LayoutQueryValues
 *                 LayoutSetValues
 *                 VioWrtCharStr
 *                 VioScrollUp
 *                 VioSetCurType
 *
 * Required files: To build and run this sample code the following files
 *                 are needed:
 *                 - query.C
 *                 - query.DEF
 *                 - query.MAK
 *                 - query.LNK
 *
 *  Copyright (C) 1991 IBM Corporation
 *
 *      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 *      sample code created by IBM Corporation. This sample code is not
 *      part of any standard or IBM product and is provided to you solely
 *      for  the purpose of assisting you in the development of your
 *      applications.  The code is provided "AS IS", without
 *      warranty of any kind.  IBM shall not be liable for any damages
 *      arising out of your use of the sample code, even if they have been
 *      advised of the possibility of such damages.                                                    *
 ************************************************************************/
#define INCL_BASE
#define INCL_DOSERRORS
#define INCL_DOSMEMMGR

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

/* To include the neccessary header files for Layout Transform */
#include "layout.h"

VOID Initialize(VOID);
VOID SetAndQuery (VOID);
VOID SetCur(INT);
PVOID Allocate(ULONG);

/* Define the parameter to Hide and Restore the cursor */
#define HIDE   -1
#define RESTORE 0

/* Define the buffer for the string to be displayed */
CHAR cString[55];

/* Defines for the LayoutTransformText  */
LAYOUT_OBJECT          hlo;
LAYOUT_VALUES          layout[4];
LAYOUT_TEXT_DESCRIPTOR  descr;
PLAYOUT_TEXT_DESCRIPTOR Pdescr;


/*************************************************************************
 * Name          : Main
 *
 * Description   : This is the main part of the program, it first clears
 *                 the screen, then hides the cursor.
 *
 * API's         : NlsSetBidiAtt
 *                 LayoutCreateObject
 *                 LayoutDestroyObject
 *                 VioScrollUp
 *
 * Parameters    : This program does not take any command line parameters
 *
 * Returns       : None
 *
 ************************************************************************/

INT main(SHORT sArgc,CHAR **ppArgv)
{
   APIRET RC,Rc;
   CHAR   msg[50];
   ULONG  len;

   /* clear the screen  */
   VioScrollUp(0, 0, 24, 79, -1," \x07", 0);

   /* hide the cursor */
   SetCur(HIDE);

   /* Create the Layout Object */

   RC=LayoutCreateObject (Locale_Arabic,&hlo);

   if (RC)
   {
   /* Get a message from the message file QUERY.msg */
     Rc = DosGetMessage(0,0, msg, sizeof(msg),
                        0,"QUERY.msg",&len);
     if (Rc)
      {
        printf("DosGetMessage Error RC=%d\n",Rc);
        exit(1);
       }
    msg[len]=0;
    printf("\n\n%s %d\n",msg, RC);
    exit(1);
    }

   Initialize();

   /* Call the API to manipulate the string and write it to the screen */
   SetAndQuery();

   /* Wait for Enter key to end the program */

    Rc = DosGetMessage(0,0, msg, sizeof(msg),
                       9,"QUERY.msg",&len);
    if (Rc)
     {
       printf("DosGetMessage Error RC=%d\n",Rc);
       exit(1);
      }
   msg[len]=0;
   VioWrtCharStr(msg, 32, 23, 26, 0l);
   getchar();

   /* clear the screen before exit */
   VioScrollUp(0, 0, 24, 79, -1," \x07", 0);

   /* restore the cursor position */
   SetCur(RESTORE);

   /* Free the Layout Object */
   LayoutDestroyObject(hlo);

   return (0);
}
/*************************************************************************
 * Name          : INITIALIZE
 *
 * Description   : This function saves the current Bidi attributes of
 *                 the session in a global variable, then sets session with
 *                 its own Bidi attributes to protect the displayed shaped
 *                 string from been processed again according to the session
 *                 Bidi attributes.
 *                 Then the function displays some static text and also the
 *                 String to be converted before calling the API.
 *
 * API's         : NlsQueryBidiAtt
 *                 NlsSetBidiAtt
 *                 VioWrtCharStr
 *
 * Parameters    : This program does not take any command line parameters
 *
 * Returns       : None
 *
 ************************************************************************/
VOID Initialize(VOID)
{
   INT iI, iRow, iCol;

   APIRET  RC,Rc;
   CHAR    msg[50];
   ULONG   len;

   /* define structures to be arrays of strings to contain the static
    * text to be wirtten on the screen  */
   struct {
          CHAR cScreen[40];
          }mssg1[3];

   struct {
           CHAR cScreen[45];
           }mssg2[3];

   /* Put a title for the example */
   Rc = DosGetMessage(0,0, cString, sizeof(cString),
                      3,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   cString[len]=0;
   VioWrtCharStr(cString, strlen(cString), 2, 21, 0l);

   Rc = DosGetMessage(0,0, cString, sizeof(cString),
                      4,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   cString[len]=0;
   VioWrtCharStr(cString, strlen(cString), 4, 21, 0l);

   Rc = DosGetMessage(0,0, cString, sizeof(cString),
                      5,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   cString[len]=0;
   VioWrtCharStr(cString, strlen(cString), 5, 21, 0l);

   /* Initialize the structures with the characters to be displayed */

   Rc = DosGetMessage(0,0, mssg1[0].cScreen, sizeof(mssg1[0].cScreen),
                      6,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg1[0].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg1[1].cScreen, sizeof(mssg1[1].cScreen),
                      7,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg1[1].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg1[2].cScreen, sizeof(mssg1[2].cScreen),
                      8,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg1[2].cScreen[len]=0;

   /* In this for loop, we increment the iRow after the display of each
    * string in the structure to jump to another line, keeping the value
    * of the iCol value fixed to have the strings indented */
   for(iI = 0, iRow = 7, iCol = 21; iI < 3; iI++, iRow++)
     VioWrtCharStr(mssg1[iI].cScreen, strlen(mssg1[iI].cScreen),
                   iRow, iCol, 0l);


   /* Initialize the second structure with the characters to be displayed */

   Rc = DosGetMessage(0,0, mssg2[0].cScreen, sizeof(mssg2[0].cScreen),
                      10,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg2[0].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg2[1].cScreen, sizeof(mssg2[1].cScreen),
                      11,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg2[1].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg2[2].cScreen, sizeof(mssg2[2].cScreen),
                      12,"QUERY.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg2[2].cScreen[len]=0;

   for(iI = 0, iRow = 15, iCol = 21; iI <= 2; iI++, iRow++)
     VioWrtCharStr(mssg2[iI].cScreen, strlen(mssg2[iI].cScreen),
                   iRow, iCol, 0l);

}
/*************************************************************************
 * Name          : SetAndQuery
 *
 * Description   : This function will show how to use the Bidi API
 *                 LayoutQueryValues
 *
 * API's         : LayoutQueryValues
 *                 LayoutSetValues
 *                 VioWrtCharStr
 *
 * Parameters    : This program does not take any command line parameters
 *
 * Returns       : None
 *
 ************************************************************************/
VOID SetAndQuery(VOID)
{
   APIRET  RC,Rc;
   ULONG   index;
   BOOL    CallerAllocate;
   ULONG   AttrSetIn, AttrSetOut;
   CHAR    msg[50];
   ULONG   len;



   /* Define the LayoutValues that will need changing */
   layout[0].name= TypeOfText|Orientation|Numerals|TextShaping;
   layout[0].value=&descr;


   /* using the OR operator, we set the bits in the Layout Descriptor,
    * to describe the input string attributes, it is RTL, contains
    * Arabic numerals, and it is in Base shapes */

   descr.in=TEXT_VISUAL|ORIENTATION_RTL|NUMERALS_NOMINAL|TEXT_NOMINAL;


   /* Now let's compose the Text Descriptor for the output string to be
    * LTR, Hindu numeral, and Shaped (Automatic Shape Determination) */

   descr.out=TEXT_VISUAL|ORIENTATION_LTR|NUMERALS_NATIONAL|TEXT_SHAPED;

   layout[1].name= CallerAllocMem;

   layout[1].value=&CallerAllocate;

   CallerAllocate = TRUE ;

   layout[2].name=0;           /* End of change */

     /* Set the LayoutValues   */
     RC=LayoutSetValues (hlo,layout,&index);


     if (RC)
     {
        Rc = DosGetMessage(0,0, msg, sizeof(msg),
                           13,"QUERY.msg",&len);
        if (Rc)
         {
           printf("DosGetMessage Error RC=%d\n",Rc);
           exit(1);
          }
        msg[len]=0;
        printf("%s %ld\n",msg,index);
        exit(0);
      }

     AttrSetIn   = descr.in;
     AttrSetOut  = descr.out;

  /* calling the API.. to Query the setting */

       Pdescr = (PLAYOUT_TEXT_DESCRIPTOR)Allocate(sizeof(LAYOUT_TEXT_DESCRIPTOR));
       layout[0].name= TypeOfText|Orientation|Numerals|TextShaping;
       layout[0].value =Pdescr;
       layout[1].name=0;           /* End of change */

       RC = LayoutQueryValues(hlo,layout,&index);

       Pdescr = layout[0].value;

       /* If there is an error returned */
       if(RC != 0)
       {
        Rc = DosGetMessage(0,0, msg, sizeof(msg),
                           14,"QUERY.msg",&len);
        if (Rc)
         {
           printf("DosGetMessage Error RC=%d\n",Rc);
           exit(1);
          }
        msg[len]=0;
        printf("%s %ld\n",msg,RC);
        exit(1);
       }
       if ((AttrSetIn == Pdescr->in ) && (AttrSetOut == Pdescr->out)) {
          Rc = DosGetMessage(0,0, cString, sizeof(cString),
                             15,"QUERY.msg",&len);
          if (Rc)
           {
             printf("DosGetMessage Error RC=%d\n",Rc);
             exit(1);
            }
          cString[len]=0;
          VioWrtCharStr(cString, strlen(cString), 20, 21, 0l);
       } /* endif */

       DosFreeMem(Pdescr);

}

/*************************************************************************
 * Name          : SET_CUR
 *
 * Description   : This function hides or restores the cursor position.
 *
 * API's         : VioSetCurType
 *
 * Parameters    : (INT flag)
 *                 flag indicates whether we want ot hide or restore
 *                 the cursor.
 *
 * Returns       : None
 *
 ************************************************************************/

VOID SetCur(INT iFlag)
{
   HVIO HVIOhandle = 0;
   VIOCURSORINFO cursor;

   cursor.yStart  = 3;
   cursor.cEnd    = 5;
   cursor.cx      = 0;
   cursor.attr    = iFlag;

   VioSetCurType(&cursor, HVIOhandle);
}
/***************************************************************************\
* Allocate
*
*
* This function allocates the required amount of memory.
*
* returns: PVOID - Pointer to allocated memory
*          NULL  - Allocation failed
*
\***************************************************************************/
PVOID Allocate(ULONG Size)
{
  PVOID Pointer;
                           /* If the memory allocation is successful */
    if (DosAllocMem(&Pointer,Size,PAG_READ | PAG_WRITE | PAG_COMMIT)==0)
                         /* Return a pointer to the allocated memory */
      return(Pointer);
    else                                    /* Return a null pointer */
      return(NULL);
}

